from dataclasses import dataclass, field
import typing
import datetime as dt

@dataclass
class Pomodoro:
    work_time: int
    break_time: int
    start_time: typing.Optional[dt.datetime] = field(default=None,init=False)
    pomo_end_time: typing.Optional[dt.datetime] = field(default=None,init=False)
    break_end_time: typing.Optional[dt.datetime] = field(default=None,init=False)
    pomo_number: int = field(default=1,init=False)
    started: bool = field(default=False,init=False)

    def start(self):
        work_secs = self.work_time * 60
        break_secs = self.break_time * 60

        self.start_time = self._get_time()
        self.pomo_end_time = self.start_time + dt.timedelta(seconds=work_secs)
        self.break_end_time = self.pomo_end_time + dt.timedelta(seconds=break_secs)
        self.started = True

    def get_state(self) -> str:
        if self.started == False:
            return "init"

        now = self._get_time()

        if now < self.pomo_end_time:
            return "pomo"
        elif now <= self.break_end_time:
            return "break"
        else:
            return "done"

    def restart(self):
        self.pomo_number += 1
        self.start()

    def _get_time(self) -> dt.datetime:
        return dt.datetime.now()



"""
Gui classs: handles printing to the screen and user input
"""
import curses
from typing import Tuple

class Gui:

    def __init__(self,screen):
        curses.curs_set(0)
        curses.use_default_colors()
        curses.init_pair(1, curses.COLOR_RED, -1)

        self.screen = screen
        self.screen.nodelay(True)

    def clear_screen(self):
        self.screen.clear()

    def beep(self):
      curses.beep()
        
    def get_time_display(self,time_text) -> str:
      a = "####" 
      b = "#  #"
      c = "#   "
      d = "   #"
      o = "   "
      x = " # "

        
                   # 0,1,2,3,4,5,6,7,8,9,:
      templates = [ [a,d,a,a,b,a,a,a,a,a,o],
                    [b,d,d,d,b,c,c,d,b,b,x],
                    [b,d,a,a,a,a,a,d,a,a,o],
                    [b,d,c,d,d,d,b,d,b,d,x],
                    [a,d,a,a,d,a,a,d,a,a,o] ]

      out = ""
      for row in templates: 
        for digit in time_text:
          if digit == ":":
            digit = 10
          else:
            digit = int(digit,10)
          out = out + row[digit] + " "
        out = out + "\n"

      return out
      
    def print_screen(self,text):
        lines = text.rstrip("\n").split("\n")
        y,x = self.get_center_start(lines)
        self.display_lines(lines,y,x)

    def get_center_start(self, lines: list[str]) -> Tuple[int,int]:
        x = 0
        y = 0

        num_rows, num_cols = self.screen.getmaxyx()
        middle_row = int(num_rows / 2)
        middle_col = int(num_cols / 2)

        longest_line = max(map(len,lines))

        y = middle_row - int(len(lines) / 2)
        x = middle_col - int(longest_line / 2)

        return y,x

    def display_lines(self, lines: list[str], y: int, x: int):
      for line in lines:
        self.screen.addstr(y,x,line + "\n", curses.color_pair(1))
        y = y + 1
      self.screen.refresh()

    def handle_screen_input(self):
      try:
        key = self.screen.getkey()
      except curses.error as e:
        if str(e) == 'no input': return ''
        raise e
      return key



import unittest
import curses
import pathlib


from pypomodoro.gui import Gui

class TestGui(unittest.TestCase):
    
    def setUp(self):
        self.debug_text = None

        self.screen = curses.initscr()
        curses.start_color()
        self.gui = Gui(self.screen)

    def tearDown(self):
        curses.endwin()
        if self.debug_text:
            print("\n\n" + self.debug_text + "\n")

    def test_get_time_display(self):
        
        # Get expected output
        display_out_file = pathlib.Path(__file__).parent.joinpath('data/display_text_out.txt')
        with open(display_out_file, 'r') as f:
            expected_display_text = f.read()

        time_display = self.gui.get_time_display("01:23:04")
        self.assertEqual(time_display, expected_display_text)

    def test_get_center_start(self):
        # Text: 3 lines, longest line is 20chars 
        text = "aaaaaaaaaaaaaaaaaaaa\nbbbbbbbbbbbbb\naaaaaaaaaaaa"

        num_rows, num_cols = self.screen.getmaxyx()
        middle_row = int(num_rows / 2)
        middle_col = int(num_cols / 2)

        expected_y = middle_row - int(3/2)
        expected_x = middle_col - int(20/2)

        center_start = self.gui.get_center_start(text.splitlines())

        self.assertEqual(center_start, (expected_y, expected_x))



#!/usr/bin/env python3
"""
Pomodoro
"""

__author__ = "Andrew Lowe"
__version__ = "0.7.1"
__license__ = "MIT"

import curses
import argparse
import time
import datetime as dt

from .pomodoro import Pomodoro
from .gui import Gui

def init_args():
  """ This is executed when run from the command line """
  parser = argparse.ArgumentParser()
  parser.add_argument("-w", "--work", default=25, type=float, action="store", dest="work_mins", help="Number of miinutes for work")
  parser.add_argument("-b", "--break", default=5, type=float, action="store", dest="break_mins", help="Number of minutes for break")
  parser.add_argument("-v", "--verbose", action="store_true", help="Move detailed display")
  
  # Specify output of "--version"
  parser.add_argument(
      "--version",
      action="version",
      version="%(prog)s (version {version})".format(version=__version__))

  args = parser.parse_args()
  return args

def start(screen):
    state_params = {
      'pomo' : {
        'text': 'Time to work',
        'alerted': False
      },
      'break': {
        'text': 'Take a break',
        'alerted': False
      },
      'done': {
        'text': 'Press space to get back to work',
        'alerted': False
      },
      'init': {
        'text': 'Press space to start work',
      }
    }

    time_left_init = "00:00:00"

    args = init_args()

    pomo = Pomodoro(args.work_mins,args.break_mins)
    gui = Gui(screen)

    gui.clear_screen()

    while True:
      pom_state = pomo.get_state()

      key_pressed = gui.handle_screen_input()

      if key_pressed == 'q':
        exit(0);

      now = dt.datetime.now()
      pom_end_time = pomo.pomo_end_time
      break_end_time = pomo.break_end_time
      pom_start_time = pomo.start_time

      if pom_state == "init":
        time_left = time_left_init
        if key_pressed == ' ':
          pomo.start()
          continue
      elif pom_state == "pomo":
        time_left = pom_end_time - now
      elif pom_state == "done":
        time_left = time_left_init
        if key_pressed == ' ':
          for st in state_params:
            state_params[st]['alerted'] = False
          pomo.restart()
          continue
      else :
        time_left = break_end_time - now

      display_info = state_params[pom_state]['text']

      if key_pressed != '':
        display_info = "Press 'q' to quit"
      
      if args.verbose and pom_state != "init":
        display_info += "\n" + f"Pomo number: {pomo.pomo_number}"
        display_info += "\n" + f"Pomo end time: {pomo.pomo_end_time:%H:%M:%S}"
        display_info += "\n" + f"Break end time: {pomo.break_end_time:%H:%M:%S%z}"
      time_left = str(time_left).split(".")[0]
      time_left = "{:0>8}".format(time_left)
      display_text = gui.get_time_display(time_left)
      display_text = display_text + "\n" + display_info

      if pom_state != 'init' and state_params[pom_state]['alerted'] == False:
        gui.beep()
        state_params[pom_state]['alerted'] = True
      gui.print_screen(display_text)
      time.sleep(1)

def main(screen=None):
  
  if not screen:
      curses.wrapper(main)
  else:
    start(screen)

if __name__ == "__main__": main()



import pathlib
from setuptools import setup, find_packages

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="pypomodoro",
    version="0.7.1",
    description="Displays a full screen pomodoro timer in the terminal",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/lostways/pypomodoro",
    author="Andrew Lowe",
    author_email="andrew@lostways.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    packages=find_packages(),
    entry_points = {
      'console_scripts': ['pypomodoro=pypomodoro.main:main']
    },
)




#!/usr/bin/env python3

import unittest
import datetime as dt

from freezegun import freeze_time

from pypomodoro.pomodoro import Pomodoro

class TestPomodoro(unittest.TestCase):

    def test_init(self):
        pom = Pomodoro(work_time=10,break_time=5)
        self.assertEqual(pom.pomo_number,1)
        self.assertEqual(pom.started,False)
        self.assertEqual(pom.start_time,None)
        self.assertEqual(pom.pomo_end_time,None)
        self.assertEqual(pom.break_end_time,None)
        self.assertEqual(pom.work_time,10)
        self.assertEqual(pom.break_time,5)

    def test_start(self):
        WORK_TIME = 10
        BREAK_TIME =5

        pom = Pomodoro(work_time=WORK_TIME,break_time=BREAK_TIME)

        with freeze_time(dt.datetime.now()):
            pom.start()
            self.assertEqual(pom.start_time, dt.datetime.now())
            self.assertEqual(
                pom.pomo_end_time,
                dt.datetime.now() + dt.timedelta(seconds=WORK_TIME * 60)
            )
            self.assertEqual(
                pom.break_end_time,
                pom.pomo_end_time + dt.timedelta(seconds=BREAK_TIME * 60)
            )
            self.assertEqual(pom.started, True)

    def test_get_state(self):
        WORK_TIME = 10
        BREAK_TIME =5

        pom = Pomodoro(work_time=WORK_TIME, break_time=BREAK_TIME)
        self.assertEqual(pom.get_state(), "init")

        with freeze_time(dt.datetime.now()) as frozen_time:
            pom.start()
            self.assertEqual(pom.get_state(), "pomo")
            
            frozen_time.tick((WORK_TIME * 60) + 1)
            self.assertEqual(pom.get_state(), "break")

            frozen_time.tick((WORK_TIME * 60) + (BREAK_TIME * 60))
            self.assertEqual(pom.get_state(), "done")

    def test_restart(self):
        WORK_TIME = 10
        BREAK_TIME =5

        pom = Pomodoro(work_time=WORK_TIME, break_time=BREAK_TIME)
        self.assertEqual(pom.get_state(), "init")

        pom.start()
        self.assertEqual(pom.get_state(), "pomo")
        self.assertEqual(pom.pomo_number, 1)

        with freeze_time(dt.datetime.now()):
            pom.restart()

            self.assertEqual(pom.get_state(), "pomo")
            self.assertEqual(pom.pomo_number, 2)

            self.assertEqual(pom.start_time, dt.datetime.now())
            self.assertEqual(
                pom.pomo_end_time,
                dt.datetime.now() + dt.timedelta(seconds=WORK_TIME * 60)
            )
            self.assertEqual(
                pom.break_end_time,
                pom.pomo_end_time + dt.timedelta(seconds=BREAK_TIME * 60)
            )
            self.assertEqual(pom.started, True)

if __name__ == '__main__':
    unittest.main()
